'''
python script for Charlotte Tomkin's project
- parsing data for King Edward I's itinerary
- creating map images
- putting these into a gif 
'''


'''
DEPENDENCIES
- created on Ubuntu linux 14.06
- assumes bash environment & ImageMagick for making gif
- python assumes conda environment contained in mapping.yml
'''

import pandas as pd #dataframes
import os #file and folder functions
import numpy as np
import datetime
import matplotlib.pyplot as plt #plotting function
import cartopy.crs as ccrs
import cartopy
import cartopy.feature as cfeature

#os.chdir('/home/tom/t.stafford@sheffield.ac.uk/A_UNIVERSITY/toys/kingedwardI/code')

#load data
print("We assume this script is in a subfolder /code")
print("...and that there is another subfolder /data")
df=pd.read_csv('../data/itinerary.csv') #nb day and year should be ints not floats
df_locs=pd.read_csv('../data/locations.csv')

#load castle locations and dates
cf=pd.read_csv('../data/castles.csv')
cf.drop("Unnamed: 6",axis=1,inplace=True) #drop unused column
cf.columns=['castle','country','lat','long','date','notes'] #rename columns
cf['year']=cf['date'].apply(lambda x:int(str(x).replace('s','').replace(' ',''))) #clean date info

bf=pd.read_csv('../data/borders.csv')
bf['lat']=pd.to_numeric(bf['latitude'],errors='coerce')
bf['long']=pd.to_numeric(bf['longitude'],errors='coerce')

#add lat and long info
df=df.merge(df_locs,how='left',on=['Place','County'])

df.rename(columns={'longitude': 'lon', 'latitude': 'lat'}, inplace=True)

#tidy up by dropping entries which don't have data for
print("you could be losing data at this point from badly formed input data")
df.dropna(axis=0,subset=['year','lon','lat'],inplace=True)

#add explicit sequence info
print("you should add explicitly sorting by date here")
df['seq']=range(len(df)) #NB assumes already sorted by date

'''
#extract essential fields
#place,lat,lon,arr,dep
df['place']=df['events'].apply(lambda x: x['mapKey'])
df['lat']=df['events'].apply(lambda x: x['LatLong'][0] if x['LatLong'] else np.NaN)
df['lon']=df['events'].apply(lambda x: x['LatLong'][1] if x['LatLong'] else np.NaN)
df['arr']=df['events'].apply(lambda x: x['start'] if x['start'] else np.NaN)
df['dep']=df['events'].apply(lambda x: x['end'] if 'end' in x else np.NaN)

#calculate dwell time if there is an explicit departure date
def getdats(row):
    try:
        #return (pd.to_datetime(row['arr'],format='%d/%m/%Y',errors='ignore')-pd.to_datetime(row['dep'],format='%d/%m/%Y',errors='ignore'))
        return datetime.datetime.strptime(row['dep'], '%Y-%m-%d')-datetime.datetime.strptime(row['arr'], '%Y-%m-%d')
    except:
        return np.NaN

df['dwell']=df.apply(getdats,axis=1).dt.days


#tidy up
df.drop('events',axis=1,inplace=True) #drop original import single column
df.sort_values('arr',inplace=True) #arrange by arrival date
df['seq']=range(len(df)) #explicitly label sequence of itinery, in case we rearrange df later

#calculate departure date from arrival date at next place, less one day
df['arr-1']=df['arr'].shift(-1) #make arrival at next place explicit as new column
df['dep-1']=df['arr-1'].apply(lambda x: datetime.datetime.strptime(x, '%Y-%m-%d')-datetime.timedelta(days=1) if pd.notnull(x) else np.NaN) #calculate arrival at next place less one day

df['dwell2']=df['dep-1']-df['arr'].apply(lambda x: datetime.datetime.strptime(x, '%Y-%m-%d'))

df['dwell2']=df['dwell2'].apply(lambda x: x.days) #convert to int
'''

'''
--------------------------> MAKING A MAP
'''
# Cribbing from
# https://uoftcoders.github.io/studyGroup/lessons/python/cartography/lesson/
# and
# https://matthewkudija.com/blog/2018/04/21/travel-map-cartopy/




def new_base(df):
    plt.clf()
    # map boundaries

    vert_buffer=(df.lat.max()-df.lat.min())*0.05
    horz_buffer=(df.lon.max()-df.lon.min())*0.25
    top = df.lat.max()+vert_buffer
    bottom = df.lat.min()-vert_buffer
    left = df.lon.min()-horz_buffer
    right = df.lon.max()-horz_buffer
    #override
    #top=56.60;bottom=43.15;left=-8.22;right=1.83
    top=58.8;bottom=48.8566;left=-8;right=3.9
            
    ax = plt.axes(projection=ccrs.PlateCarree())
    
    # [lon_min, lon_max, lat_min, lat_max]
    ax.set_extent([left, right, bottom, top], crs=ccrs.PlateCarree())
    
    # add and color high-resolution land
    OCEAN_highres = cfeature.NaturalEarthFeature('physical', 'ocean', '50m',
                                                 edgecolor='#B1B2B',
                                                 facecolor='lightblue',
                                                 linewidth=.1
                                                 )
    
    LAND_highres = cfeature.NaturalEarthFeature('physical', 'land', '50m',
                                                 facecolor='green',
                                                 linewidth=.1
                                             )    

    #This works, sort of, but doesn't add much at this schale
    RIVERS_highres = cfeature.NaturalEarthFeature('physical', 'rivers_lake_centerlines', '50m',
                                                  linewidth=0.1)
    
    #ax.add_feature(OCEAN_highres, zorder=0,
    #               edgecolor='#B1B2B4', facecolor='#7f7f7f')
    
    ax.add_feature(LAND_highres,zorder=0,facecolor='lightgreen')
    
    ax.add_feature(OCEAN_highres, zorder=0,
                   edgecolor='#B1B2B4', facecolor='lightblue')
                   
    ax.add_feature(RIVERS_highres,facecolor='None',edgecolor='blue')
    
    add_borders(bf)
    
    
def add_text():
    px=0.05;p2=px+0.1;p3=p2+0.025
    py=0.1
    plt.annotate('Itinerary of King Edward I', xy=(px,0.94),size=8, xycoords='axes fraction',color='black')
    plt.annotate('peace', xy=(px,py),size=5, xycoords='axes fraction',color='blue')
    plt.annotate('/', xy=(p2,py),size=5, xycoords='axes fraction',color='black')
    plt.annotate('war', xy=(p3,py),size=5, xycoords='axes fraction',color='red')
    plt.annotate('Charlotte Tomkins, @tomstafford, @Pseudo_Isidore',xy=(px,0.03),size=5, xycoords='axes fraction',color='black')


def add_castles(cf,year):
    #custome markers 
    # https://stackoverflow.com/questions/2318288/how-to-use-custom-marker-with-plot
    # https://stackoverflow.com/questions/22566284/matplotlib-how-to-plot-images-instead-of-points/53851017
    mask=cf['year']<=year
    plt.plot(cf[mask]['long'],cf[mask]['lat'],'x',color='k',ms=2.5)
    #plt.plot(cf[mask]['long'],cf[mask]['lat'],marker=r'$\boxtimes$',lw=0,color='k',ms=6) #don't work
    
    
def add_borders(bf):
    plt.plot(bf['long'],bf['lat'],'x',ms=1,color='white')
        
            


markparam = 14
year=0

alphaval=1

colorbase=1199

xpos=[0.05]*16+[0.2]*15
ypos=np.linspace(0.4,0.1,16)


cumulative=False

'''
--------------------------> ADDING ITINERARY DETAILS
'''

new_base(df)

for index, row in df.iterrows():
    if index < 10000000:

        lastyear=year
        year=int(row['year'])
        try:
            nextyear=int(df[df['seq']==(row['seq']+1)]['year'])
        except:
            if df['seq'].max()==row['seq']:
                nextyear=year+1                
        
        colorname='blue' #our_colors[(year-11)%18]
        
        filename = '../frames/' + str(index).rjust(5, '0') + '.png'

        marksize = markparam * np.log(1 + 0.5)
        
        if row['peace or war']=='War':
            colorname='red'
        else:
            colorname='blue'
            
        plt.plot(row['lon'], row['lat'], '.', alpha=alphaval, ms=marksize, color=colorname)


        add_text()
        
        add_castles(cf,year)                
        
        if (not nextyear==year):
            annotext=str(year)
            print("Making : " + annotext)
            #ann = plt.annotate(annotext, xy=(xpos[(year+3)%32], ypos[(year+3)%16]),xycoords='axes fraction',color=colorname)
            ann = plt.annotate(annotext, xy=(0.8, 0.94),xycoords='axes fraction',color=colorname,size=8)
            plt.savefig(filename,dpi=320,bbox_inches='tight')
            ann.remove()
            if not cumulative:
                new_base(df)
                                
        # plt.title(annotext)
        # plt.ylim([46,56])
        # plt.xlim([-4,2])
        
    
'''
--------------------------> ANIMATING
'''




# combine frames into gif
if cumulative:
    os.system('convert -delay 130 ../frames/*.png ../edward_itineraryCUMULATIVE.gif')
    os.system('convert -delay 130 ../frames/*.png -resize "1480x1246!" ../edward_itineraryCUMULATIVE.mp4')
else:
    os.system('convert -delay 130 ../frames/*.png ../edward_itineraryYEARBYYEAR.gif')
    os.system('convert -delay 130 ../frames/*.png -resize "1480x1246!" ../edward_itineraryYEARBYYEAR.mp4')
    
'''
--------------------------> DISTANCE GRAPH
'''    

import geopy.distance


for index,rows in df.iterrows():
    try:
        coords_1 = (lastrow['lat'], lastrow['lon'])
        coords_2 = (rows['lat'],rows['lon'])
        lastrow=rows
        df.loc[index,'dist']=(geopy.distance.distance(coords_1, coords_2).miles)
    except Exception as e: 
        print("Exception: "+ e)
        lastrow=rows
        
    
    
plt.clf()    
df.groupby('year').dist.sum().plot(marker='.')
plt.ylabel('Total distance traveled (as crow flies), miles')
plt.savefig('../travel_by_year.png',dpi=180)

'''
df.groupby('month').dist.sum().plot()
plt.ylabel('Total distance traveled (as crow flies), miles')
plt.savefig('travel_by_year.png',dpi=180)
'''

def monthcounter(row):
    r=row.lower()[:3]
    if r=='jan':
        n=1
    elif r=='feb':
        n=2
    elif r=='mar':
        n=3
    elif r=='apr':
        n=4
    elif r=='may':
        n=5;
    elif r=='jun':
        n=6
    elif r=='jul':
        n=7
    elif r=='aug':
        n=8
    elif r=='sep':
        n=9
    elif r=='oct':
        n=10
    elif r=='nov':
        n=11
    elif r=='dec':
        n=12
    else:
        n=np.NaN
    return n
        
    
df['month_n']=df['month'].apply(monthcounter)

plt.clf()
df.groupby('month_n').dist.sum().plot()
plt.xlabel('Month')
plt.ylabel('Total distance travelled')
plt.savefig('../travel.png')



